/* SPIM S20 MIPS simulator.
   Code to read a MIPS a.out file.
   Copyright (C) 1990 by James Larus (larus@cs.wisc.edu).

   SPIM is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 1, or (at your option) any
   later version.

   SPIM is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.

   You should have received a copy of the GNU General Public License
   along with GNU CC; see the file COPYING.  If not, write to James R.
   Larus, Computer Sciences Department, University of Wisconsin--Madison,
   1210 West Dayton Street, Madison, WI 53706, USA or to the Free
   Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */


/* $Header: /var/home/larus/SPIM/RCS/read-aout.c,v 3.3 90/12/11 08:36:44 larus Exp Locker: larus $
*/

#ifdef mips
#include <a.out.h>
#include <stdio.h>
#include <ldfcn.h>
#endif mips

#include "spim.h"
#include "inst.h"
#include "mem.h"


/* Exported function: */

int read_aout_file ();


/* Imported functions: */

void data_begins_at_point ();
instruction *inst_decode ();
void store_instruction ();
void store_word ();
void text_begins_at_point ();


/* Exported variables: */
int errno_addr = 0;		/* Address of errno in data segment */
int program_break;		/* Last address in data segment (edata) */


/* Imported variables: */

mem_addr program_starting_address;
int text_dir;
extern int errno;



/* Read a MIPS executable file.  Return zero if successful and
   non-zero otherwise. */

int
read_aout_file (file_name)
     char *file_name;
{
#ifdef mips
  AOUTHDR header;
  FILE *fd;
  register int i;
  int text_base, text_size;
  int data_base, data_size;

  LDFILE *ldptr;
  PDR ppd;
  SYMR sym;

  bare_machine = 1;		/* Before storing into memory */
  source_file = 0;

  fd = fopen (file_name, "r");
  if (fd == NULL)
    {
      error ("Cannot read the file: %s\n", file_name);
      return (1);
    }
  /* skip over the filehdr: */
  fseek (fd, sizeof (struct filehdr), 0);
  /* read the aouthdr: */
  if (fread (&header, 1, sizeof (AOUTHDR), fd) != sizeof (AOUTHDR))
    {
      error ("File had invalid format: %s\n", file_name);
      return (1);
    }

  /* a.out preface is read into core before text segment: */
  text_base = header.text_start +  sizeof (struct filehdr) + sizeof (AOUTHDR);
  text_size = header.tsize - (sizeof (struct filehdr) + sizeof (AOUTHDR));

  text_begins_at_point (text_base);
  text_dir = 1;
  for (i = 0;i < text_size / 4; i ++)
    store_instruction (inst_decode (getw (fd)));
  text_dir = 0;
  if (feof (fd))
    {
      error ("Cannot read entire text segment: %s\n", file_name);
      return (1);
    }
  program_starting_address = header.entry;

  /* a.out preface is read into core before text segment: */
  data_base = header.data_start;
  data_size = header.dsize;

  program_break = header.bss_start + header.bsize;

  data_begins_at_point (data_base);
  for (i = 0;i < data_size / 4; i ++)
    store_word (getw (fd));
  if (feof (fd))
    error ("Cannot read entire data segment: %s\n", file_name);


  if ((ldptr = ldopen (file_name, NULL)) == NULL)
    error ("Cannot read file %s for errno address\n", file_name);
  else
    {
      if (SYMTAB(ldptr) != NULL)
	for (i = 0; i < SYMHEADER (ldptr).isymMax+SYMHEADER(ldptr).iextMax;
	     i++)
	  {
	    char *name;

	    ldtbread (ldptr, i, &sym);
	    name = (char *) ldgetname (ldptr, &sym);

	    if (streq(name, "errno") && sym.st == stGlobal)
	      {
		errno_addr = sym.value;
		break;
	      }
	  }
      if (errno_addr == 0)
	error("Cannot find `errno' in %s's symbol table\n", file_name);
      ldclose(ldptr);
    }
#endif

  return (0);
}
